--
-- PlayerStateActivateWorkshop
--
-- @author Rockstar
-- @date 26/05/2021
--

--
-- @fs22 22/11/2021
--


PlayerStateActivateWorkshop = {}
local PlayerStateActivateWorkshop_mt = Class(PlayerStateActivateWorkshop, PlayerStateBase)

function PlayerStateActivateWorkshop.new(player, stateMachine)
	local self = PlayerStateBase.new(player, stateMachine, PlayerStateActivateWorkshop_mt)

	self.vehicle = nil
	self.placeable = nil

	self.interactText = ""

	self.isDealer = false
	self.isOwnWorkshop = false
	self.isMobileWorkshop = true

	g_messageCenter:subscribe(SellPlaceableEvent, self.onPlaceableSellEvent, self)

	return self
end

function PlayerStateActivateWorkshop:delete()
	PlayerStateActivateWorkshop:superClass().delete(self)

	g_messageCenter:unsubscribeAll(self)
end

function PlayerStateActivateWorkshop:isAvailable()
	self.vehicle = nil
	self.placeable = nil

	if self.player:getIsInputAllowed() then
		local lastFoundObject = self.player.lastFoundWorkshopObject

		if lastFoundObject ~= nil then
			local object = g_currentMission:getNodeObject(lastFoundObject)

			if object ~= nil and object.isa ~= nil then
				if object:isa(Placeable) then
					if object:getDestructionMethod() ~= Placeable.DESTRUCTION.PER_NODE then
						self.placeable = object
						self.interactText = string.format(g_i18n:getText("action_configSellSpecificObject"), object:getName())

						return true
					end
				elseif object:isa(Vehicle) then
					local isPallet = object.typeName == "pallet"
					local isRidable = SpecializationUtil.hasSpecialization(Rideable, object.specializations)

					if not isRidable and not isPallet and object.getSellPrice ~= nil and object.price ~= nil then
						self.vehicle = object
						self.interactText = string.format("%s: %s", g_i18n:getText("action_configSellSpecificVehicle"), object:getFullName())

						return true
					end
				end
			end
		end
	end

	self.interactText = ""

	return false
end

function PlayerStateActivateWorkshop:activate()
	PlayerStateActivateWorkshop:superClass().activate(self)

	if self.placeable ~= nil then
		g_gui:showPlaceableInfoDialog({
			placeable = self.placeable
		})
	elseif self.vehicle ~= nil then
		local vehicles = {}
		local childVehicles = self.vehicle.rootVehicle:getChildVehicles()

		for i = 1, #childVehicles do
			local childVehicle = childVehicles[i]

			if g_currentMission.accessHandler:canPlayerAccess(childVehicle, self.player) then
				table.addElement(vehicles, childVehicle)
			end
		end

		table.sort(vehicles, function (a, b)
			return a.rootNode < b.rootNode
		end)

		g_workshopScreen:setSellingPoint(nil, self.isDealer, self.isOwnWorkshop, self.isMobileWorkshop)
		g_workshopScreen:setVehicles(vehicles)
		g_workshopScreen.list:setSelectedIndex(table.findListElementFirstIndex(vehicles, self.vehicle))
		g_gui:showGui("WorkshopScreen")
	end

	self:deactivate()
end

function PlayerStateActivateWorkshop:onPlaceableSellEvent(errorCode, sellPrice)
	if errorCode == SellPlaceableEvent.STATE_SUCCESS then
		g_currentMission:showMoneyChange(MoneyType.SHOP_PROPERTY_SELL)
	end
end