--
-- main
--
-- @author Rockstar
-- @date 26/05/2021
--

--
-- @fs22 22/11/2021
--


source(g_currentModDirectory .. "scripts/Player/PlayerStateActivateWorkshop.lua")

local function PlayerStateMachineNew_inj(self, superFunc, player, custom_mt)
	local ret = superFunc(self, player, custom_mt)

	ret.playerStateActivateWorkshop = PlayerStateActivateWorkshop.new(ret.player, ret)
	ret.stateList.activateWorkshop = ret.playerStateActivateWorkshop
	ret.fsmTable.crouch.activateWorkshop = true
	ret.fsmTable.idle.activateWorkshop = true
	ret.fsmTable.activateWorkshop = {
		crouch = true,
		idle = true,
		walk = true,
		run = true
	}

	return ret
end

local function PlayerNew_inj(self, superFunc, isServer, isClient)
	local ret = superFunc(self, isServer, isClient)

	ret.inputInformation.registrationList[InputAction.ACTIVATE_WORKSHOP] = {
		text = "",
		triggerAlways = false,
		triggerDown = true,
		eventId = "",
		textVisibility = true,
		triggerUp = false,
		callback = function (targetObject, actionName, inputValue, callbackState, isAnalog, isMouse, deviceCategory, binding)
			if targetObject.playerStateMachine:isAvailable("activateWorkshop") then
				targetObject.playerStateMachine:activateState("activateWorkshop")
			end
		end,
		activeType = Player.INPUT_ACTIVE_TYPE.IS_MOVEMENT
	}

	return ret
end

local function PlayerUpdateActionEvents_inj(self)
	local stateActivateWorkshop = false

	if not self:hasHandtoolEquipped() then
		if self.playerStateMachine:isAvailable("activateWorkshop") then
			local eventIdActivateWorkshop = self.inputInformation.registrationList[InputAction.ACTIVATE_WORKSHOP].eventId
			local activateWorkshopState = self.playerStateMachine:getState("activateWorkshop")

			g_inputBinding:setActionEventText(eventIdActivateWorkshop, activateWorkshopState.interactText)
			stateActivateWorkshop = true
		end
	end

	self:setInputState(InputAction.ACTIVATE_WORKSHOP, stateActivateWorkshop)
end

local function PlayerCheckObjectInRange_inj(self)
	if self.isControlled then
		if not self.isCarryingObject then
			self.lastFoundWorkshopObject = nil
		end
	end
end

local function PlayerPickUpObjectRaycastCallback_inj(self, superFunc, hitObjectId, x, y, z, distance)
	if hitObjectId ~= g_currentMission.terrainRootNode and Player.PICKED_UP_OBJECTS[hitObjectId] ~= true then
		self.lastFoundWorkshopObject = hitObjectId
	end

	return superFunc(self, hitObjectId, x, y, z, distance)
end

PlayerStateMachine.new = Utils.overwrittenFunction(PlayerStateMachine.new, PlayerStateMachineNew_inj)
Player.new = Utils.overwrittenFunction(Player.new, PlayerNew_inj)
Player.updateActionEvents = Utils.appendedFunction(Player.updateActionEvents, PlayerUpdateActionEvents_inj)
Player.checkObjectInRange = Utils.prependedFunction(Player.checkObjectInRange, PlayerCheckObjectInRange_inj)
Player.pickUpObjectRaycastCallback = Utils.overwrittenFunction(Player.pickUpObjectRaycastCallback, PlayerPickUpObjectRaycastCallback_inj)